/*
 * Line.java
 *
 * Modified on 3. 12. 2008
 * 
 */
package cz.muni.fi.pb162.traffic.network;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Common line of mass traffic system. The vehicles drive throw stations there
 * and back again (linear line) or make a circle (belt line).
 * 
 * @author oprendek
 */
public final class Line {

    private String no;
    private boolean beltLine;
    private List<Stop> stops = new ArrayList<Stop>();

    /**
     * Creates new line specified by symbol
     * @param no symbol labeling the line
     * @throws NullPointerException if symbol is null
     * @throws IllegalArgumentException if symbol is an empty string
     */
    public Line(String no, boolean beltLine) {
        if (no == null) {
            throw new NullPointerException("Line symbol can not be null");
        }
        if ("".equals(no)) {
            throw new IllegalArgumentException("Line symbol can not be empty");
        }
        this.no = no;
        this.beltLine = beltLine;
    }

    /**
     * Adds a stop behind the last stop on the line
     * @param stop stop to add
     * @throws NullPointerException if stop is null
     */
    public void addStop(Stop stop) {
        if (stop == null) {
            throw new NullPointerException("parameter stop");
        }
        stops.add(stop);
    }

    /**
     * Adds a stop before the first occurence of specified stop
     * @param stop stop to add
     * @param before stop that should be before the inserted stop
     * @throws NullPointerException if stop is null
     * @throws NullPointerException if before is null
     * @throws IllegalArgumentException if line does not contain stop "before"
     */
    public void addStop(Stop before, Stop stop) {
        if (stop == null) {
            throw new NullPointerException("parameter stop");
        }
        //prva varianta
        if (!stops.contains(before)) {
            throw new IllegalArgumentException("stop 'before' not in line");
        }
        stops.add(stops.indexOf(before), stop);

        //druha varianta
        final int index = stops.indexOf(before);
        if (index < 0) {
            throw new IllegalArgumentException("stop 'before' not in line");
        }
        stops.add(index, stop);
    }

    /**
     * Removes the first occurence of the specified stop
     * @param stop stop to remove
     * @throws NullPointerException if stop is null
     * @throws IllegalArgumentException if line does not contain stop
     */
    public void removeStop(Stop stop) {
        if (stop == null) {
            throw new NullPointerException("param. stop");
        }
        if (!stops.contains(stop)) {
            throw new IllegalArgumentException("stop not in list");
        }
        stops.remove(stop);

    }

    //doprogramovat na hodine
    /**
     * Lists stops in common order (as visited by the vehicle by drive there)
     * @return stop listing
     */
    public String listStops() {
        StringBuilder stringBuilder = new StringBuilder();
        for (Stop stop : stops) {
            stringBuilder.append(stop);
            stringBuilder.append(", ");
        }
        return stringBuilder.subSequence(0, stringBuilder.length() - 2).toString();
    }

    /**
     * Indicates whether the line containsStop a stop or does not
     * 
     * @param stop stop to check
     * @return <CODE>true</CODE> if the line containsStop that stop,<br><CODE>false</CODE> otherwise
     */
    public boolean containsStop(Stop stop) {
        return stops.contains(stop);
    }

    /**
     * Returns a hash code value for the object computed from the "no" parameter.
     *
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        return no.hashCode();
    }

    /**
     * Indicates whether some other line is "equal to" this one, what means, has the same "no"
     *
     * @param obj   the reference object with which to compare.
     * @return <code>true</code> if this object has the same no as the obj
     *          argument; <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof Line)) {
            return false;
        }
        return ((Line) obj).getNo().equals(no);
    }

    /**
     * Returns the symbol of the line
     * @return line's symobl
     */
    public String getNo() {
        return no;
    }

    /**
     * Finds stops you can travel on this line by one hop. 
     * @param stop stop at hop zero (start)
     * @return a set of stops with distance 1 from start stop
     */
    Set<Stop> nextStops(Stop stop) {

        Set<Stop> nextStops = new HashSet<Stop>();
        int stopIndex = 0;
        for (Stop lineStop : stops) {
            if (lineStop.equals(stop)) {

                if (beltLine) {
                    nextStops.add(stops.get((stopIndex + 1) % stops.size()));
                } else {
                    if (stopIndex + 1 < stops.size()) {
                        nextStops.add(stops.get(stopIndex + 1));
                    }
                    if (stopIndex - 1 >= 0) {
                        nextStops.add(stops.get(stopIndex - 1));
                    }
                }
            }
            stopIndex++;
        }
        return nextStops;
    }

    /**
     * Gives the basic information about the line
     * @return text information about the line
     */
    @Override
    public String toString() {
        return "Line " + no;
    }
}
