/*
 * Network.java
 *
 * Modified on 3. 12. 2008
 * 
 */
package cz.muni.fi.pb162.traffic.network;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;

/**
 * Mass traffic network. Implements the singleton design pattern.
 * @author misko
 */
public class Network {

    private static final Network SINGLETON = new Network();
    Set<Line> lines = new HashSet<Line>();

    private Network() {
    }

    /**
     * Gives the network. Implements the singleton design pattern -- only one network is to work with
     * @return mass traffic network
     */
    public static Network getInstance() {
        return SINGLETON;
    }

    /**
     * Adds a line to the network
     * @param line line to add
     * @throws NullPointerException if line is null
     */
    public void addLine(Line line) {
        if (line == null) {
            throw new NullPointerException("Argument line can not be null");
        }
        lines.add(line);
    }

    /**
     * Removes the line from the network
     * @param line line to remove
     * @throws NullPointerException if line is null
     * @throws IllegalArgumentException if network does not contain line
     */
    public void removeLine(Line line) {
        if (line == null) {
            throw new NullPointerException("Argument line can not be null");
        }
        if (lines.contains(line)) {
            lines.remove(line);
        } else {
            throw new IllegalArgumentException("Network does not contain this line");
        }
    }

    /**
     * Finds stops you can travel by one hop from specified stop on all lines. 
     * @param stop stop at hop zero (start)
     * @return a set of stops with distance 1 from start stop
     */
    Set<Stop> getNextStops(Stop stop) {

        Set<Stop> nextStops = new HashSet<Stop>();

        for (Line line : lines) {
            if (line.containsStop(stop)) {
                nextStops.addAll(line.nextStops(stop));
            }
        }
        return nextStops;
    }

    /**
     * Computes the shortes distance between two stops in the network
     * @param s1 first stop
     * @param s2 second stop
     * @return shortest distance between first and second stop if there is, -1 otherwise
     */
    public int distance(Stop s1, Stop s2) {
        int distance = 0;
        if (s1.equals(s2)) {
            return distance;
        }
        Map<Stop, Integer> processed = new HashMap<Stop, Integer>();
        Queue<Stop> queue = new LinkedList<Stop>();
        queue.add(s1);
        processed.put(s1, 0);
        do {

            Stop stop = queue.poll();
            distance = processed.get(stop);
            for (Stop nextStop : getNextStops(stop)) {
                if (nextStop.equals(s2)) {
                    return distance + 1;
                }
                if (!processed.keySet().contains(nextStop)) {
                    queue.add(nextStop);
                    processed.put(nextStop, distance + 1);
                }
            }
        } while (!queue.isEmpty());
        return -1;
    }

    /**
     * Gives the basic information about the network
     * @return text information about the line
    
     */
    public String toString() {
        return "Network containing lines " + lines;
    }


    /**
     * Reinitialisation -- discarts all lines
     * @deprecated only for test purposes
     */
    public void reset() {
        lines.clear();
    }
    
}
