/*
 * Ticket.java
 *
 * Modified on 3. 12. 2008
 */
package cz.muni.fi.pb162.traffic.tickets;

import cz.muni.fi.pb162.traffic.network.Stop;

/**
 * Pre-paid ticket for traveling. One unit of credit is used to travel between 
 * two adjacent stops.
 * @author misko
 */
public final class Ticket {

    private int credit;
    private int id;
    private Stop enterPoint = null;

    /**
     * Creates new mass trafic prepaid ticket with specified id and start credit
     * @param id numeric identifier of ticket
     * @param credit start credit on the ticket
     * @throws IllegalArgumentException if credit is negative
     */
    Ticket(int id, int credit) {
        if (credit < 0) {
            throw new IllegalArgumentException("Credit can not be negative");
        }
        this.id = id;
        this.credit = credit;
    }

    /**
     * Gives amount of credit on the ticket
     * @return current amount of credit
     */
    public int getCredit() {
        return credit;
    }

    /**
     * Informs whether the ticket is checket at the moment or in not
     * @return <CODE>true</CODE> if the ticket is checked,<br/><CODE>false</CODE> otherwise
     */
    public boolean isChecked() {
        return enterPoint != null;
    }

    /**
     * Checks the ticket by the entering the transport area
     * @param stop stop where the transport area is entered
     * @throws NullPointerException if stop is null
     * @throws IllegalStateException if ticket was not checked-out
     */
    public void checkIn(Stop stop) {
        if (stop == null) {
            new NullPointerException("Station can not be null");
        }
        if (isChecked()) {
            throw new IllegalStateException("allready checked in");
        }
        enterPoint = stop;
    }

    /**
     * Unchecks the ticket by leaving the transport area
     * @return stop where the transport area is entered
     * @throws IllegalStateException if ticket was not checked-in
     */
    public Stop checkOut() {

        if (!isChecked()) {
            throw new IllegalStateException("not checked in");
        }
        Stop result = enterPoint;
        enterPoint = null;
        return result;
    }

    /**
     * Decreases credit on the ticket by the count of travelet stops
     * @param stops count of traveled stops
     * @throws IllegalArgumentException if the count of traveled stops is negative
     */
    public void subtractCredit(int stops) {
        if (stops < 0) {
            throw new IllegalArgumentException("Can not travel throw negative count of stations");
        }
        credit -= stops;
    }

    /**
     * Adds bought credit to the ticket
     * @param boughtCredit amount of bought credit to add
     * @throws IllegalArgumentException if the amount of bought credit is negative
     */
    public void addCredit(int boughtCredit) {
        if (boughtCredit < 0) {
            throw new IllegalArgumentException("Can not buy (add) negative amount of credit");
        }
        credit += boughtCredit;
    }
}
